@@ -36,9 +36,17 @@ class CrcCheck(enum.Enum):
     #: :class:`fitdecode.FitCRCError` will ever be raised.
     READONLY = 1
 
+    #: CRC is computed and `FitReader` emits a warning message via
+    #: `warnings.warn()` in case of mismatching CRC value. No
+    #: :class:`fitdecode.FitCRCError` will ever be raised.
+    WARN = 2
+
     #: CRC is computed and matched by `FitReader`.
-    #: :class:`fitdecode.FitCRCError` is raised upon incorrect CRC values.
-    ENABLED = 2
+    #: :class:`fitdecode.FitCRCError` is raised upon incorrect CRC value.
+    RAISE = 3
+
+    #: Alias of `RAISE` for backward compatibility.
+    ENABLED = RAISE
 
 
 class ErrorHandling(enum.Enum):
@@ -56,8 +64,8 @@ class ErrorHandling(enum.Enum):
     #: `FitDataMessage` frames that rely on this definition.
     IGNORE = 0
 
-    #: Default behavior. Same as `DISABLED` but `FitReader` emits a warning with
-    #: `warnings.warn`.
+    #: Default behavior. Same as `IGNORE` but `FitReader` emits a warning with
+    #: `warnings.warn()`.
     WARN = 1
 
     #: Strict mode. `FitReader` raises a `FitParseError` exception when
@@ -138,12 +146,12 @@ class FitReader:
     """
 
     def __init__(
-            self, fileish, *, processor=_UNSET, check_crc=CrcCheck.ENABLED,
+            self, fileish, *, processor=_UNSET, check_crc=CrcCheck.WARN,
             error_handling=ErrorHandling.WARN, keep_raw_chunks=False,
             data_bag=_UNSET):
         # backward compatibility
         if check_crc is True:
-            check_crc = CrcCheck.ENABLED
+            check_crc = CrcCheck.RAISE
         elif check_crc is False:
             check_crc = CrcCheck.DISABLED
 
@@ -166,10 +174,11 @@ class FitReader:
             self._processor = processor
         self._keep_raw = keep_raw_chunks
 
-        # state (private)
+        # per-stream state (private)
         self._fd = None        # the file object to read from
         self._read_offset = 0  # read cursor position in the file
         self._read_size = 0    # count bytes read from this file so far in total
+        self._fit_file_index = -1  # the index of the current FIT file in this data stream  # noqa
 
         # per-chunk state (private)
         self._chunk_index = 0   # the index number of the current chunk that is currently being read  # noqa
@@ -245,6 +254,21 @@ class FitReader:
         """The last read ``file_id`` `FitDataMessage` object. May be `None`."""
         return self._file_id
 
+    @property
+    def fit_file_index(self):
+        """
+        The zero-based index `int` of the so-called *FIT file* currently read
+        from this data stream so far.
+
+        `None` if no *FIT file* header has been encoutered yet.
+        """
+        return self._fit_file_index if self._fit_file_index >= 0 else None
+
+    @property
+    def fit_files_count(self):
+        """The number of FIT files found in this data stream so far."""
+        return self._fit_file_index + 1
+
     @property
     def local_mesg_defs(self):
         """
@@ -299,31 +323,35 @@ class FitReader:
     def _read_next(self):
 
         def _update_state():
-            if self._fd:
+            if self._fd is not None:
                 self._chunk_index += 1
                 self._chunk_offset += self._chunk_size
                 self._chunk_size = 0
 
-        while self._fd:
+        while self._fd is not None:
             assert self._chunk_size == 0
 
-            if not self._header:
+            if self._header is None:
                 assert self._body_bytes_left == 0
 
-                self._on_new_file()
+                self._reset_per_fit_state()
                 self._read_header()
-                if not self._header:
+                if self._header is None:
                     break
 
+                self._fit_file_index += 1
+
                 yield self._header
                 _update_state()
 
             elif self._body_bytes_left > 0:
-                assert self._header
+                assert self._header is not None
 
                 record = self._read_record()
-                if not record:
-                    break
+
+                assert isinstance(record, (
+                    records.FitDefinitionMessage,
+                    records.FitDataMessage))
 
                 assert self._chunk_size <= self._body_bytes_left
                 self._body_bytes_left -= self._chunk_size
@@ -332,13 +360,13 @@ class FitReader:
                 _update_state()
 
             else:
-                assert self._header
+                assert self._header is not None
                 assert self._body_bytes_left == 0
 
                 try:
                     crc_obj = self._read_crc()
                 except FitEOFError:
-                    # if self.check_crc is not CrcCheck.ENABLED:
+                    # if self.check_crc is not CrcCheck.RAISE:
                     #     # There is no CRC footer in this file (or it is
                     #     # incomplete) but caller does not mind about CRC so
                     #     # we will just ignore this
@@ -352,10 +380,10 @@ class FitReader:
                 # behavior due to malformed FIT stream (i.e. next FIT header
                 # missing), reset the internal state now as well, instead of
                 # resetting it only when a FIT header is read.
-                self._on_new_file()
+                self._reset_per_fit_state()
 
-    def _on_new_file(self):
-        # reset state
+    def _reset_per_fit_state(self):
+        # reset per-FIT-file state
         self._crc = utils.CRC_START
         self._header = None
         self._body_bytes_left = 0
@@ -375,11 +403,11 @@ class FitReader:
                 # regular EOF: storage is empty or previous "FIT file" ended
                 # normally
                 return
-            raise FitHeaderError(f'file too small ({exc})')
+            raise FitHeaderError(f'file truncated? {exc}')
 
         # check header size
         if header_size < len(chunk) or header_magic != b'.FIT':
-            raise FitHeaderError('not a FIT file')
+            raise FitHeaderError(f'not a FIT file @ {self._chunk_offset}')
 
         # read the extended part of the header (i.e. byte 12-...)
         extra_header_size = header_size - len(chunk)
@@ -388,11 +416,15 @@ class FitReader:
         if extra_header_size:
             # at least 2 bytes expected for the CRC
             if extra_header_size < 2:
-                raise FitHeaderError('unsupported FIT header')
+                raise FitHeaderError(
+                    f'unsupported FIT header (CRC field missing; '
+                    f'header offset: {self._chunk_offset})')
 
             extra_chunk = self._read_bytes(extra_header_size)
             if len(extra_chunk) != extra_header_size:
-                raise FitHeaderError('truncated FIT header')
+                raise FitHeaderError(
+                    f'truncated FIT header '
+                    f'(header offset: {self._chunk_offset})')
 
             (read_crc, ) = struct.unpack('<H', extra_chunk)
             if not read_crc:  # can be null according to SDK
@@ -401,8 +433,18 @@ class FitReader:
             else:
                 computed_crc = utils.compute_crc(chunk)
                 crc_matched = computed_crc == read_crc
-                if self.check_crc is CrcCheck.ENABLED and not crc_matched:
-                    raise FitCRCError('invalid FIT header CRC')
+
+                if not crc_matched and (
+                        self.check_crc is CrcCheck.WARN or
+                        self.check_crc is CrcCheck.RAISE):
+                    msg = (
+                        f'mismatching CRC in FIT header '
+                        f'(header offset: {self._chunk_offset})')
+
+                    if self.check_crc is CrcCheck.RAISE:
+                        raise FitCRCError(msg)
+                    elif self.check_crc is CrcCheck.WARN:
+                        warnings.warn(msg)
 
             chunk += extra_chunk
 
@@ -427,8 +469,19 @@ class FitReader:
         computed_crc = self._crc
         chunk, read_crc = self._read_struct('<H')
 
-        if self.check_crc is CrcCheck.ENABLED and computed_crc != read_crc:
-            raise FitCRCError()
+        if computed_crc != read_crc and (
+                self.check_crc is CrcCheck.WARN or
+                self.check_crc is CrcCheck.RAISE):
+            msg = (
+                f'mismatching CRC in FIT footer '
+                f'(footer offset: {self._chunk_offset}; '
+                f'read crc: {read_crc}; '
+                f'expected crc: {computed_crc})')
+
+            if self.check_crc is CrcCheck.RAISE:
+                raise FitCRCError(msg)
+            elif self.check_crc is CrcCheck.WARN:
+                warnings.warn(msg)
 
         crc_obj = records.FitCRC(
             read_crc,
